//	Background.cpp - Class representing the background watercolour splodges.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "Background.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "Watercolours.h"
#include "Random.h"

//------------------------------------------------------------------------------
uint32_t Background::images[6] = {0, 0, 0, 0, 0, 0};

//------------------------------------------------------------------------------
Background::Background():
globalFade(0.0f),
alpha(1.0f)
{
	int i, j, k;
	bool tooClose;

	j = (int)(Random::getGlobal().getRandomFloat() * 5.99f);

	for(i=0;i<6;++i)
	{
		//Set splodge position.
		tooClose = true;
		while(tooClose)
		{
			splodges[i].position.x = 128.0f + (Random::getGlobal().getRandomFloat() * (1280.0f-256.0f));
			splodges[i].position.y = 128.0f + (Random::getGlobal().getRandomFloat() * (720.0f-256.0f));
			tooClose = false;
			for(k=(i-1);k>=0;--k)
			{
				if(splodges[i].position.getDistance(splodges[k].position) < 256.0f)
				{
					tooClose = true;
					break;
				}
			}
		}

		//Set splodge scale.
		splodges[j].scale = (float)(i/6.0f);

		//Set splodge rotation.
		splodges[i].rotation = Random::getGlobal().getRandomFloat() * 359.99f;

		//Set splodge rotation speed.
		splodges[i].rotSpeed = ((Random::getGlobal().getRandomFloat() * 2.0f) - 1.0f) * 0.125f;

		//Set splodge image.
		splodges[i].image = i;

		++j;
		j %= 6;
	}
}

//------------------------------------------------------------------------------
Background::~Background()
{

}

//------------------------------------------------------------------------------
void Background::draw(Drawer *d)
{
	int i;
	float scale;
	float tempAlpha;

	if(!images[0])
	{
		ImageLoader imLoad;
		ImageData data;

		data = imLoad.loadImage(Watercolours::watercolour0_png, Watercolours::watercolour0_pngSize);
		images[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Watercolours::watercolour1_png, Watercolours::watercolour1_pngSize);
		images[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Watercolours::watercolour2_png, Watercolours::watercolour2_pngSize);
		images[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Watercolours::watercolour3_png, Watercolours::watercolour3_pngSize);
		images[3] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Watercolours::watercolour4_png, Watercolours::watercolour4_pngSize);
		images[4] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Watercolours::watercolour5_png, Watercolours::watercolour5_pngSize);
		images[5] = d->loadImage(data.data, data.width, data.height);
	}

	for(i=0;i<6;++i)
	{
		scale = (splodges[i].scale*0.5f) + 0.5f;
		tempAlpha = 1.0f-((cosf(splodges[i].scale * 6.283185f) * 0.5f)+0.5f);

		d->drawImageRot(images[splodges[i].image],
						splodges[i].position.x - (scale * 512.0f),
						splodges[i].position.y - (scale * 512.0f),
						splodges[i].rotation,
						512.0f,
						512.0f,
						scale,
						scale,
						1.0f,
						1.0f,
						1.0f,
						tempAlpha * globalFade * alpha);

		splodges[i].scale += 0.001f;
		if(splodges[i].scale > 1.0f)
		{
			int k;
			bool tooClose;

			splodges[i].scale -= 1.0f;

			tooClose = true;
			while(tooClose)
			{
				splodges[i].position.x = 128.0f + (Random::getGlobal().getRandomFloat() * (1280.0f-256.0f));
				splodges[i].position.y = 128.0f + (Random::getGlobal().getRandomFloat() * (720.0f-256.0f));
				tooClose = false;
				for(k=(i-1);k>=0;--k)
				{
					if(splodges[i].position.getDistance(splodges[k].position) < 256.0f)
					{
						tooClose = true;
						break;
					}
				}
			}

			splodges[i].rotSpeed = ((Random::getGlobal().getRandomFloat() * 2.0f) - 1.0f) * 0.125f;
		}

		splodges[i].rotation += splodges[i].rotSpeed;
		splodges[i].rotation = fmodf(splodges[i].rotation, 360.0f);
	}

	if(globalFade < 1.0f)
		globalFade += 0.01f;
}

//------------------------------------------------------------------------------
void Background::setAlpha(float val)
{
	alpha = val * 0.5f;
}
