//	Background.h - Class representing the background watercolour splodges.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef BACKGROUND_H_
#define BACKGROUND_H_

#include "TwoFloats.h"

#include <stdint.h>

class Drawer;

///	Class representing the background watercolour splodges.
class Background
{
  public:
	///	Constructor.
	Background();
	///	Destructor.
	~Background();

	///	Draws the splodges.
	void draw(Drawer *d);

	///	Used to fade the background out.
	void setAlpha(float val);
  private:
	///	Struct representing a single splodge.
	struct Splodge
	{
		///	The splodge's position.
		TwoFloats position;
		///	The splodge's scale.
		float scale;
		///	The splodge's rotation.
		float rotation;
		///	The splodge's speed of rotation.
		float rotSpeed;
		///	The splodge's image.
		int image;
	};

	///	The 6 watercolour images.
	static uint32_t images[6];

	///	The 6 splodges.
	Splodge splodges[6];

	///	Used to fade the splodges in upon construction.
	float globalFade;

	///	Used to fade the splodes out.
	float alpha;
};

#endif
