//	GamePlay.cpp - Main gameplay state.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "GamePlay.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "Random.h"
#include "Images.h"
#include "synth/SynthSingleton.h"
#include "MiscHelper.h"

#include "agents/Trail.h"
#include "agents/Letter.h"
#include "agents/Word.h"

using namespace std;

//------------------------------------------------------------------------------
uint32_t GamePlay::treeImage = 0;
uint32_t GamePlay::starImage = 0;
uint32_t GamePlay::starBlurImage = 0;
uint32_t GamePlay::noiseImage = 0;
uint32_t GamePlay::earthImage = 0;
uint32_t GamePlay::starImages[6] = {0, 0, 0, 0, 0, 0};
uint32_t GamePlay::bigFont = 0;
uint32_t GamePlay::weeFont = 0;

//------------------------------------------------------------------------------
const TwoFloats startPoints[227] = {
							 TwoFloats(649, 400),
							 TwoFloats(699, 395),
							 TwoFloats(696, 302),
							 TwoFloats(672, 137),
							 TwoFloats(585, 166),
							 TwoFloats(587, 216),
							 TwoFloats(561, 259),
							 TwoFloats(486, 248),
							 TwoFloats(428, 300),
							 TwoFloats(480, 188),
							 TwoFloats(580, 192),
							 TwoFloats(548, 120),
							 TwoFloats(536, 102),
							 TwoFloats(449, 16),
							 TwoFloats(376, 107),
							 TwoFloats(348, 193),
							 TwoFloats(392, 140),
							 TwoFloats(350, 92),
							 TwoFloats(283, 152),
							 TwoFloats(425, 38),
							 TwoFloats(342, 8),
							 TwoFloats(249, 49),
							 TwoFloats(171, 33),
							 TwoFloats(102, 107),
							 TwoFloats(152, 185),
							 TwoFloats(97, 235),
							 TwoFloats(78, 196),
							 TwoFloats(59, 302),
							 TwoFloats(73, 427),
							 TwoFloats(136, 430),
							 TwoFloats(644, 376),
							 TwoFloats(640, 352),
							 TwoFloats(627, 334),
							 TwoFloats(611, 320),
							 TwoFloats(592, 320),
							 TwoFloats(569, 326),
							 TwoFloats(552, 334),
							 TwoFloats(539, 342),
							 TwoFloats(526, 352),
							 TwoFloats(662, 382),
							 TwoFloats(676, 395),
							 TwoFloats(688, 377),
							 TwoFloats(676, 361),
							 TwoFloats(670, 342),
							 TwoFloats(660, 324),
							 TwoFloats(678, 318),
							 TwoFloats(675, 302),
							 TwoFloats(654, 300),
							 TwoFloats(635, 292),
							 TwoFloats(620, 284),
							 TwoFloats(601, 280),
							 TwoFloats(614, 267),
							 TwoFloats(620, 248),
							 TwoFloats(630, 225),
							 TwoFloats(635, 201),
							 TwoFloats(646, 180),
							 TwoFloats(662, 166),
							 TwoFloats(675, 150),
							 TwoFloats(654, 148),
							 TwoFloats(636, 158),
							 TwoFloats(620, 172),
							 TwoFloats(611, 193),
							 TwoFloats(601, 177),
							 TwoFloats(585, 185),
							 TwoFloats(590, 203),
							 TwoFloats(585, 233),
							 TwoFloats(577, 252),
							 TwoFloats(550, 272),
							 TwoFloats(532, 281),
							 TwoFloats(512, 292),
							 TwoFloats(504, 275),
							 TwoFloats(499, 257),
							 TwoFloats(481, 267),
							 TwoFloats(480, 286),
							 TwoFloats(476, 304),
							 TwoFloats(470, 323),
							 TwoFloats(459, 304),
							 TwoFloats(440, 316),
							 TwoFloats(444, 329),
							 TwoFloats(443, 350),
							 TwoFloats(448, 283),
							 TwoFloats(462, 268),
							 TwoFloats(507, 238),
							 TwoFloats(528, 238),
							 TwoFloats(544, 248),
							 TwoFloats(564, 233),
							 TwoFloats(555, 211),
							 TwoFloats(528, 206),
							 TwoFloats(504, 211),
							 TwoFloats(480, 224),
							 TwoFloats(459, 217),
							 TwoFloats(560, 188),
							 TwoFloats(542, 184),
							 TwoFloats(524, 180),
							 TwoFloats(500, 185),
							 TwoFloats(568, 168),
							 TwoFloats(552, 160),
							 TwoFloats(537, 156),
							 TwoFloats(552, 140),
							 TwoFloats(531, 136),
							 TwoFloats(512, 145),
							 TwoFloats(496, 150),
							 TwoFloats(473, 156),
							 TwoFloats(481, 132),
							 TwoFloats(502, 120),
							 TwoFloats(520, 108),
							 TwoFloats(515, 91),
							 TwoFloats(494, 96),
							 TwoFloats(473, 107),
							 TwoFloats(454, 96),
							 TwoFloats(452, 72),
							 TwoFloats(467, 48),
							 TwoFloats(464, 25),
							 TwoFloats(446, 38),
							 TwoFloats(430, 62),
							 TwoFloats(424, 86),
							 TwoFloats(398, 84),
							 TwoFloats(404, 108),
							 TwoFloats(422, 115),
							 TwoFloats(433, 131),
							 TwoFloats(436, 152),
							 TwoFloats(422, 171),
							 TwoFloats(416, 192),
							 TwoFloats(398, 184),
							 TwoFloats(376, 177),
							 TwoFloats(369, 200),
							 TwoFloats(384, 206),
							 TwoFloats(400, 222),
							 TwoFloats(400, 244),
							 TwoFloats(388, 267),
							 TwoFloats(376, 292),
							 TwoFloats(371, 313),
							 TwoFloats(344, 291),
							 TwoFloats(337, 265),
							 TwoFloats(329, 236),
							 TwoFloats(328, 206),
							 TwoFloats(334, 176),
							 TwoFloats(356, 158),
							 TwoFloats(372, 128),
							 TwoFloats(352, 132),
							 TwoFloats(339, 108),
							 TwoFloats(326, 129),
							 TwoFloats(310, 144),
							 TwoFloats(302, 160),
							 TwoFloats(286, 177),
							 TwoFloats(289, 196),
							 TwoFloats(284, 216),
							 TwoFloats(297, 132),
							 TwoFloats(310, 105),
							 TwoFloats(329, 88),
							 TwoFloats(348, 70),
							 TwoFloats(369, 59),
							 TwoFloats(396, 46),
							 TwoFloats(403, 25),
							 TwoFloats(379, 28),
							 TwoFloats(356, 38),
							 TwoFloats(339, 27),
							 TwoFloats(326, 44),
							 TwoFloats(312, 59),
							 TwoFloats(291, 51),
							 TwoFloats(270, 41),
							 TwoFloats(273, 64),
							 TwoFloats(280, 81),
							 TwoFloats(278, 94),
							 TwoFloats(265, 110),
							 TwoFloats(256, 128),
							 TwoFloats(238, 113),
							 TwoFloats(211, 102),
							 TwoFloats(190, 94),
							 TwoFloats(171, 76),
							 TwoFloats(169, 52),
							 TwoFloats(145, 56),
							 TwoFloats(147, 81),
							 TwoFloats(124, 84),
							 TwoFloats(134, 110),
							 TwoFloats(150, 120),
							 TwoFloats(168, 131),
							 TwoFloats(147, 148),
							 TwoFloats(171, 161),
							 TwoFloats(187, 148),
							 TwoFloats(212, 144),
							 TwoFloats(232, 161),
							 TwoFloats(240, 182),
							 TwoFloats(241, 200),
							 TwoFloats(236, 217),
							 TwoFloats(216, 219),
							 TwoFloats(196, 224),
							 TwoFloats(177, 235),
							 TwoFloats(168, 251),
							 TwoFloats(174, 272),
							 TwoFloats(192, 256),
							 TwoFloats(211, 246),
							 TwoFloats(225, 248),
							 TwoFloats(235, 256),
							 TwoFloats(244, 272),
							 TwoFloats(256, 296),
							 TwoFloats(232, 312),
							 TwoFloats(216, 308),
							 TwoFloats(195, 299),
							 TwoFloats(174, 288),
							 TwoFloats(150, 272),
							 TwoFloats(134, 248),
							 TwoFloats(113, 232),
							 TwoFloats(100, 249),
							 TwoFloats(118, 265),
							 TwoFloats(81, 227),
							 TwoFloats(65, 211),
							 TwoFloats(64, 241),
							 TwoFloats(75, 267),
							 TwoFloats(62, 275),
							 TwoFloats(78, 292),
							 TwoFloats(99, 292),
							 TwoFloats(121, 299),
							 TwoFloats(137, 312),
							 TwoFloats(155, 324),
							 TwoFloats(174, 334),
							 TwoFloats(148, 350),
							 TwoFloats(134, 366),
							 TwoFloats(107, 382),
							 TwoFloats(81, 400),
							 TwoFloats(100, 417),
							 TwoFloats(124, 408),
							 TwoFloats(153, 404),
							 TwoFloats(166, 385),
							 TwoFloats(188, 371),
							 TwoFloats(211, 363),
							 TwoFloats(232, 363)
							};

//------------------------------------------------------------------------------
GamePlay::GamePlay(Application *app):
GameState(app),
//introTimer(500),
//introTimer(0),
treeFade(0.0f),
titleFade(0.0f),
intro(true),
currentTrail(-1),
trailCounter(0),
currentWord(-1),
wordCounter(0),
letterBorn(false),
wordBorn(false),
trailBorn(false),
numAgentTypes(0),
endCount(7200),
waitingCount(5400),
fadeToBlack(1.0f),
endFade(1.0f),
earthMove(2.0f),
endTextFade(-4.0f)
{
	//int i;

	SynthSingleton::getInstance().setGamePlay(this);

	/*for(i=0;i<256;++i)
	{
		stars.push_back(Star());

		stars[i].position.x = Random::getGlobal().getRandomFloat() * 1280.0f;
		stars[i].position.y = (Random::getGlobal().getRandomFloat() * 1280.0f) - 256.0f;

		stars[i].size = 0.25f + (Random::getGlobal().getRandomFloat() * 0.5f);

		stars[i].colour.x = 0.5f + (Random::getGlobal().getRandomFloat() * 0.5f);
		stars[i].colour.y = 0.5f + (Random::getGlobal().getRandomFloat() * 0.5f);
		stars[i].colour.z = 0.5f + (Random::getGlobal().getRandomFloat() * 0.5f);
	}*/

	starFields[0].image = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
	starFields[0].position = TwoFloats(0.0f, 0.0f - 256.0f);
	starFields[1].image = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
	starFields[1].position = TwoFloats(0.25f * 1280.0f, 0.0f - 256.0f);
	starFields[2].image = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
	starFields[2].position = TwoFloats(0.5f * 1280.0f, 0.0f - 256.0f);
	starFields[3].image = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
	starFields[3].position = TwoFloats(0.75f * 1280.0f, 0.0f - 256.0f);
	starFields[4].image = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
	starFields[4].position = TwoFloats(0.0f, (0.5f * 720.0f) - 256.0f);
	starFields[5].image = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
	starFields[5].position = TwoFloats(0.25f * 1280.0f, (0.5f * 720.0f) - 256.0f);
	starFields[6].image = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
	starFields[6].position = TwoFloats(0.5f * 1280.0f, (0.5f * 720.0f) - 256.0f);
	starFields[7].image = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
	starFields[7].position = TwoFloats(0.75f * 1280.0f, (0.5f * 720.0f) - 256.0f);
	starFields[8].image = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
	starFields[8].position = TwoFloats(0.0f, 720.0f - 256.0f);
	starFields[9].image = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
	starFields[9].position = TwoFloats(0.25f * 1280.0f, 720.0f - 256.0f);
	starFields[10].image = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
	starFields[10].position = TwoFloats(0.5f * 1280.0f, 720.0f - 256.0f);
	starFields[11].image = (int)(Random::getGlobal().getRandomFloat() * 5.99f);
	starFields[11].position = TwoFloats(0.75f * 1280.0f, 720.0f - 256.0f);
}

//------------------------------------------------------------------------------
GamePlay::~GamePlay()
{
	SynthSingleton::getInstance().stopTree();
}

//------------------------------------------------------------------------------
void GamePlay::draw(Drawer *d)
{
	int i;
	int tempint;
	uint8_t black;

	if(!bigFont)
	{
		ImageLoader imLoad;
		ImageData data;

		data = imLoad.loadImage(Images::tree_png, Images::tree_pngSize);
		treeImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::star_png, Images::star_pngSize);
		starImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::starblur_png, Images::starblur_pngSize);
		starBlurImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::spacenoise_jpg, Images::spacenoise_jpgSize);
		noiseImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::earth_png, Images::earth_pngSize);
		earthImage = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::stars0_png, Images::stars0_pngSize);
		starImages[0] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::stars1_png, Images::stars1_pngSize);
		starImages[1] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::stars2_png, Images::stars2_pngSize);
		starImages[2] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::stars3_png, Images::stars3_pngSize);
		starImages[3] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::stars4_png, Images::stars4_pngSize);
		starImages[4] = d->loadImage(data.data, data.width, data.height);

		data = imLoad.loadImage(Images::stars5_png, Images::stars5_pngSize);
		starImages[5] = d->loadImage(data.data, data.width, data.height);

		bigFont = d->loadFont("Prociono.ttf", 56.0f);
		weeFont = d->loadFont("Prociono.ttf", 32.0f);
	}

	//Movement------------------------------------------------------------------

	if(fadeToBlack)
	{
		//Update word detection.
		if(wordCounter > 0)
		{
			--wordCounter;
			if(!wordCounter)
			{
				if(currentWord == -1)
				{
					if(tempWord.str().length() > 1)
					{
						int index;

						index = (int)(Random::getGlobal().getRandomFloat() * 226.99f);

						agents.push_back(StdSharedPtr<Word>::type(new Word(startPoints[index],
																		   tempWord.str(),
																		   ThreeFloats())));

						if(!wordBorn)
						{
							wordBorn = true;
							agentVoiceover << "Word";
							SynthSingleton::getInstance().triggerVoiceover(agentVoiceover.str());
							++numAgentTypes;
						}
					}
					else
					{
						int index;

						index = (int)(Random::getGlobal().getRandomFloat() * 226.99f);

						agents.push_back(StdSharedPtr<Letter>::type(new Letter(startPoints[index],
																			   tempWord.str()[0],
																			   ThreeFloats())));

						if(!letterBorn)
						{
							letterBorn = true;
							agentVoiceover << "Letter";
							SynthSingleton::getInstance().triggerVoiceover(agentVoiceover.str());
							++numAgentTypes;
						}
					}
				}
				tempWord.str("");
				currentWord = -1;
			}
		}

		//Move agents.
		for(i=0;i<(int)agents.size();++i)
			agents[i]->move();

		//Kill any dead agents.
		for(i=((int)agents.size()-1);i>=0;--i)
		{
			if(agents[i]->getDead())
			{
				agents.erase(agents.begin()+i);

				if(currentTrail > i)
					--currentTrail;
			}
		}

		//Call nearby().
		calcNearby();
	}

	//Drawing.------------------------------------------------------------------

	//Draw background.
	black = (uint8_t)(fadeToBlack * 255.0f);
	//d->fillRect(0, 0, 1280, 720, black, black, black, 255);
	d->fillRect(0, 0, 1280, 480, black, black, black, 255);
	d->fillGradient(0, 480, 1280, 1280, black, black, black, 255, black, black, 255, 255);
	if(fadeToBlack > 0.0f)
	{
		if(fadeToBlack < 1.0f)
			background.setAlpha(fadeToBlack);

		background.draw(d);
	}

	//Draw tree.
	if((treeFade > 0.0f) && (fadeToBlack > 0.0f))
	{
		d->drawImage(treeImage, 0, 0, 0.75f, 0.75f, 1.0f, 1.0f, 1.0f, treeFade * fadeToBlack);
		if(treeFade < 1.0f)
			treeFade += 0.01f;
	}

	//Draw title.
	if((treeFade < 1.0f) && (titleFade < 1.0f))
		titleFade += 0.005f;
	if(titleFade > 0.0f)
	{
		d->drawText("A Song in the Void",
					bigFont,
					680,
					200,
					1024,
					1.0f,
					0.0f,
					0.0f,
					0.0f,
					titleFade);
		if((treeFade > 1.0f) && (titleFade < 1.0f))
			titleFade -= 0.01f;
	}

	//Draw agents.
	if((titleFade < 1.0f) && (fadeToBlack > 0.0f))
	{
		for(i=0;i<(int)agents.size();++i)
			agents[i]->draw(d, weeFont, fadeToBlack);
	}

	//Draw stars.
	if(fadeToBlack < 1.0f)
	{
		d->setBlendMode(Drawer::Additive);
		for(i=0;i<12;++i)
		{
			tempint = starFields[i].image;
			d->drawImage(starImages[tempint],
						 starFields[i].position.x,
						 starFields[i].position.y + (earthMove * 128.0f),
						 1.0f,
						 1.0f,
						 1.0f,
						 1.0f,
						 1.0f,
						 (1.0f-fadeToBlack));

			++tempint;
			tempint %= 6;
			d->drawImage(starImages[tempint],
						 starFields[i].position.x,
						 starFields[i].position.y + (earthMove * 128.0f),
						 1.0f,
						 1.0f,
						 1.0f,
						 1.0f,
						 1.0f,
						 (1.0f-fadeToBlack));
		}
		for(i=0;i<(int)stars.size();++i)
		{
			d->drawImage(starImage,
						 stars[i].position.x,
						 stars[i].position.y + (earthMove * 128.0f),
						 stars[i].size,
						 stars[i].size,
						 stars[i].colour.x,
						 stars[i].colour.y,
						 stars[i].colour.z,
						 (1.0f-fadeToBlack));
			d->drawImage(starBlurImage,
						 stars[i].position.x,
						 stars[i].position.y + (earthMove * 128.0f),
						 stars[i].size,
						 stars[i].size,
						 stars[i].colour.x,
						 stars[i].colour.y,
						 stars[i].colour.z,
						 (1.0f-fadeToBlack));
		}
		d->drawImage(noiseImage,
					 0,
					 -304 + (earthMove * 152.0f),
					 (1280.0f/1024.0f),
					 (1280.0f/1024.0f),
					 1.0f,
					 1.0f,
					 1.0f,
					 0.5f * (1.0f-fadeToBlack));

		d->setBlendMode(Drawer::AlphaBlend);
		d->drawImage(earthImage, 0, 480 + (fakeExp(earthMove*0.5f) * 2.0f * 480));

		if(endTextFade > 0.0f)
		{
			d->drawText("So raise your voice",
						bigFont,
						200.0f,
						360.0f,
						1024.0f,
						1.0f,
						1.0f,
						1.0f,
						1.0f,
						endTextFade);
		}
	}

	//Used to stop sending events to currentTrail when the player's not
	//doing anything with the mouse.
	if(trailCounter > 0)
		--trailCounter;
	else if(currentTrail > -1)
	{
		agents[currentTrail]->endPlayerMove();
		currentTrail = -1;
	}

	//Used to start counting down to the end of the story.
	if(endCount && (treeFade >= 1.0f) && (titleFade <= 0.0f))
	{
		--endCount;

		if(!endCount)
			SynthSingleton::getInstance().triggerVoiceover("Outro");
	}

	//Used to play a 'waiting' hint voiceover.
	if(waitingCount)
	{
		--waitingCount;

		if(!waitingCount && (numAgentTypes < 2))
			SynthSingleton::getInstance().triggerVoiceover("Waiting");
	}

	if((fadeToBlack > 0.0f) && (fadeToBlack < 1.0f))
		fadeToBlack -= 0.01f;
	if(fadeToBlack < 0.0f)
		fadeToBlack = 0.0f;

	if((fadeToBlack <= 0.0f) &&(earthMove > 0.0f))
		earthMove -= 0.001f;

	if((earthMove <= 0.0f) && (endTextFade < 1.0f))
		endTextFade += 0.01f;

	if(endFade < 1.0f)
	{
		endFade -= 0.005f;

		d->fillRect(0, 0, 1280, 720, 0, 0, 0, (uint8_t)((1.0f-endFade) * 255.0f));

		if(endFade <= 0.0f)
		{
			d->endDrawing();
			quit();
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::keyDown(const string& action, const SDL_keysym& keyPressed)
{
	if(action == "Escape")
	{
		if((titleFade >= 1.0f) || (treeFade < 1.0f))
		{
			treeFade += 0.01f;
			titleFade -= 0.01f;
			SynthSingleton::getInstance().startTree();
			SynthSingleton::getInstance().cancelVoiceover();
			intro = false;
		}
		else if(endCount > 0)
		{
			endCount = 0;
			SynthSingleton::getInstance().triggerVoiceover("Outro");
		}
		else
			quit();
	}
#ifdef DEBUG
	else if(keyPressed.sym == SDLK_F12)
	{
		SynthSingleton::getInstance().setGamePlay(0);
		nextState("StartPosEditor");
	}
#endif
	else if(!intro)
	{
		int value = (int)(keyPressed.sym);

		//Letters.
		if((value > 96) && (value < 123))
		{
			if(currentWord < 0)
			{
				tempWord << (char)value;

				if(tempWord.str().length() > 1)
				{
					int index;

					index = (int)(Random::getGlobal().getRandomFloat() * 226.99f);

					agents.push_back(StdSharedPtr<Word>::type(new Word(startPoints[index],
																	   tempWord.str(),
																	   ThreeFloats())));
					currentWord = agents.size()-1;

					if(!wordBorn)
					{
						wordBorn = true;
						agentVoiceover << "Word";
						SynthSingleton::getInstance().triggerVoiceover(agentVoiceover.str());
						++numAgentTypes;
					}
				}

				wordCounter = 30;
			}
			else
			{
				if(currentWord < (int)agents.size())
					agents[currentWord]->playerAddChar((char)value);
				else
				{
					while(currentWord >= (int)agents.size())
						--currentWord;

					agents[currentWord]->playerAddChar((char)value);
				}
				wordCounter = 30;
			}
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::mouseMove(int posX, int posY)
{
	if(!intro)
	{
		if(currentTrail == -1)
		{
			agents.push_back(StdSharedPtr<Trail>::type(new Trail()));
			currentTrail = agents.size()-1;
			agents[currentTrail]->playerMove(TwoFloats(posX, posY));
			trailCounter = 30;

			if(!trailBorn)
			{
				trailBorn = true;
				agentVoiceover << "Trail";
				SynthSingleton::getInstance().triggerVoiceover(agentVoiceover.str());
				++numAgentTypes;
			}
		}
		else if(trailCounter > 0)
		{
			agents[currentTrail]->playerMove(TwoFloats(posX, posY));
			trailCounter = 30;
		}
	}
}

//------------------------------------------------------------------------------
void GamePlay::addLetter(const TwoFloats& pos,
						 char letter,
						 const ThreeFloats& col)
{
	agents.push_back(StdSharedPtr<Letter>::type(new Letter(pos, letter, col)));
}

//------------------------------------------------------------------------------
void GamePlay::addWord(const TwoFloats& pos,
					   const std::string& word,
					   const ThreeFloats& col)
{
	if(word.length() > 0)
		agents.push_back(StdSharedPtr<Word>::type(new Word(pos, word, col)));
}

//------------------------------------------------------------------------------
void GamePlay::markerCallback(const string& marker)
{
	int i;
	int star;

	if(marker == "there was a tree")
		treeFade += 0.01f;
	else if(marker == "she started to sing")
	{
		SynthSingleton::getInstance().startTree();
		titleFade -= 0.01f;
		intro = false;
	}
	else if(marker == "in an instant")
	{
		SynthSingleton::getInstance().killMusic();
		fadeToBlack -= 0.01f;
		earthMove -= 0.001f;

		//Add stars for every agent.
		for(i=0,star=(int)stars.size();i<(int)agents.size();++i,++star)
		{
			stars.push_back(Star());

			stars[star].position = agents[i]->getPosition();
			stars[star].position.y -= 256.0f;

			if(agents[i]->getName() == "Letter")
				stars[star].size = 0.25f;
			else  if(agents[i]->getName() == "Trail")
				stars[star].size = 0.5f;
			else  if(agents[i]->getName() == "Word")
				stars[star].size = 1.0f;

			stars[star].colour = agents[i]->getColour();
		}

		waitingCount = 0;
	}
	else if(marker == "sing for her")
		SynthSingleton::getInstance().startCacophony();
}

//------------------------------------------------------------------------------
void GamePlay::setEnd()
{
	endFade -= 0.005f;
}

//------------------------------------------------------------------------------
void GamePlay::calcNearby()
{
	int i, j;
	const float nearDist = 128.0f;

	for(i=0;i<(int)agents.size();++i)
	{
		for(j=(i+1);j<(int)agents.size();++j)
		{
			if(agents[i]->getPosition().getDistance(agents[j]->getPosition()) < nearDist)
			{
				AgentWeak agentJ(agents[j]);
				AgentWeak agentI(agents[i]);
				
				agents[i]->nearby(agentJ, this);
				agents[j]->nearby(agentI, this);
			}
		}
	}
}
