//	StartPosEditor.cpp - Editor state used to generate the start positions for
//						 Letters and Words.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "StartPosEditor.h"
#include "NiallsDrawingLib/Drawer.h"
#include "NiallsDrawingLib/ImageLoader.h"
#include "Images.h"
#include "FilePath.h"

#include <fstream>

using namespace std;

//------------------------------------------------------------------------------
uint32_t StartPosEditor::tree = 0;

//------------------------------------------------------------------------------
StartPosEditor::StartPosEditor(Application *app):
GameState(app)
{

}

//------------------------------------------------------------------------------
StartPosEditor::~StartPosEditor()
{

}

//------------------------------------------------------------------------------
void StartPosEditor::draw(Drawer *d)
{
	unsigned int i;

	if(!tree)
	{
		ImageLoader imLoad;
		ImageData data;

		data = imLoad.loadImage(Images::tree_png, Images::tree_pngSize);
		tree = d->loadImage(data.data, data.width, data.height);
	}

	d->drawImage(tree, 0, 0, 0.75f, 0.75f);

	for(i=0;i<startPoints.size();++i)
		d->fillRect(startPoints[i].x-2, startPoints[i].y-2, 4, 4, 255, 0, 0, 255);

	d->fillRect(mousePos.x-2, mousePos.y-2, 4, 4, 0, 0, 255, 255);
}

//------------------------------------------------------------------------------
void StartPosEditor::keyDown(const string& action, const SDL_keysym& keyPressed)
{
	if(action == "Escape")
	{
		savePositions();
		quit();
	}
}

//------------------------------------------------------------------------------
void StartPosEditor::mouseMove(int x, int y)
{
	mousePos.x = (float)x;
	mousePos.y = (float)y;
}

//------------------------------------------------------------------------------
void StartPosEditor::mouseDown(int button, int x, int y)
{
	startPoints.push_back(TwoFloats(x, y));
}

//------------------------------------------------------------------------------
void StartPosEditor::savePositions()
{
	unsigned int i;
	FilePath path = FilePath::getHomeDir().getChild("ASongInTheVoidStarts.cpp");
	ofstream outStream(path.getPath().c_str());

	outStream << "const TwoFloats startPoints[" << startPoints.size() << "] = {" << endl;
	for(i=0;i<startPoints.size();++i)
	{
		outStream << "\t\t\t\t\t\t\t TwoFloats(" << startPoints[i].x;
		outStream << ", " << startPoints[i].y << ")";
		if(i != (startPoints.size()-1))
			outStream << ",";
		outStream << endl;
	}
	outStream << "\t\t\t\t\t\t\t};" << endl;
}
