//	StartPosEditor.h - Editor state used to generate the start positions for
//					   Letters and Words.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2011  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef STARTPOSEDITOR_H_
#define STARTPOSEDITOR_H_

#include "GameState.h"
#include "TwoFloats.h"

#include <stdint.h>
#include <vector>

///	Editor state used to generate the start positions for Letters and Words.
class StartPosEditor : public GameState
{
  public:
	///	Constructor.
	StartPosEditor(Application *app);
	///	Destructor.
	~StartPosEditor();

	///	Draws the tree and existing start points.
	void draw(Drawer *d);
	
	///	Used to quit the app and save the current start positions.
	void keyDown(const std::string& action, const SDL_keysym& keyPressed);
	///	Updates the mouse position.
	void mouseMove(int x, int y);
	///	Adds another start point.
	void mouseDown(int button, int x, int y);

	///	Returns this state's name.
	const std::string getName() const {return "StartPosEditor";};
  private:
	///	Helper method. Saves all the start positions to a file.
	void savePositions();

	///	The tree image.
	static uint32_t tree;

	///	All the existing start points.
	std::vector<TwoFloats> startPoints;

	///	The current position of the mouse.
	TwoFloats mousePos;
};

#endif
