//	BellSynth.h - Simple bell synth based on Risset's table of partials.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef BELLSYNTH_H_
#define BELLSYNTH_H_

///	Simple bell synth based on Risset's table of partials.
class BellSynth
{
  public:
	///	Constructor.
	BellSynth();
	///	Destructor.
	~BellSynth();

	///	Returns the audio for this synth.
	/*!
		\param buffer Stereo (interleaved) buffer to write audio to.
		\param numSamples The number of samples in the current block (i.e.
		buffer will contain 2x numSamples because it's stereo.
	 */
	void getAudio(float *buffer, int numSamples);

	///	Triggers the synth.
	void trigger();

	///	Sets the synth's frequency.
	void setFrequency(float val);
	///	Sets where the synth sits in the stereo field.
	void setPan(float val);

	///	Returns whether or not this synth is active.
	bool getActive() const {return active;};
  private:
	///	Represents a single partial.
	struct Partial
	{
		///	For the sine generation.
		float x0;
		///	For the sine generation.
		float x1;
		///	For the sine generation.
		float coeff;

		///	The frequency of the partial.
		float frequency;
		///	The amplitude of the partial.
		float amplitude;
		///	The duration of the partial.
		float duration;

		///	Whether we're in the attack (false) or release (true) section of the envelope.
		bool release;
		///	Our position within the envelope (0->1).
		float ampEnv;
	};

	///	The number of Partials.
	enum
	{
		NumPartials = 11
	};

	///	The Partials representing this bell.
	Partial partials[NumPartials];

	///	The fundamental frequency of the bell.
	float frequency;
	///	Used to make each trigger a slightly different amplitude.
	float amplitude;

	///	Where the bell sits in the stereo field.
	float pan;

	///	Whether this synth is currently active or not.
	bool active;
};

#endif
