//	SoundHelpers.cpp - Some helper stuff for the audio generation.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "SoundHelpers.h"

//------------------------------------------------------------------------------
float SoundHelpers::getFrequencyForPitch(unsigned int index)
{
	const float pitches[] = {
								932.328f,
								830.609f,
								698.456f,
								622.254f,
								554.365f,
								(932.328f*0.5f),
								(830.609f*0.5f),
								(698.456f*0.5f),
								(622.254f*0.5f),
								(554.365f*0.5f)
							};

	return pitches[index%10]; //%10 to avoid memory corruption w/out-of-bounds indices.
}

//------------------------------------------------------------------------------
float SoundHelpers::getHue(const ThreeFloats& input)
{
	float retval = 0.0f;

	float themin,themax,delta;
	ThreeFloats c2;

	themin = min(input.x, min(input.y,input.z));
	themax = max(input.x, max(input.y,input.z));
	delta = themax - themin;
	c2.z = (themin + themax) / 2.0f;
	c2.y = 0.0f;
	if (c2.z > 0 && c2.z < 1.0f)
		c2.y = delta / (c2.z < 0.5f ? (2.0f*c2.z) : (2.0f-2.0f*c2.z));
	retval = 0.0f;
	if (delta > 0.0f) {
		if (themax == input.x && themax != input.y)
			retval += (input.y - input.z) / delta;
		if (themax == input.y && themax != input.z)
			retval += (2.0f + (input.z - input.x) / delta);
		if (themax == input.z && themax != input.x)
			retval += (4.0f + (input.x - input.y) / delta);
		retval*= 60.0f;
	}

	retval /= 360.0f;

	return retval;
}
