//	TreeSynth.h - A synth representing the Tree's song.
//  ----------------------------------------------------------------------------
//	This file is part of 'GameTemplate', template code for a game.
//	Copyright (C) 2012  Niall Moody
//	
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef TREESYNTH_H_
#define TREESYNTH_H_

#include <vector>

///	A synth representing the Tree's song.
class TreeSynth
{
  public:
	///	Constructor.
	TreeSynth();
	///	Destructor.
	~TreeSynth();

	///	Returns the audio for this synth.
	/*!
		\param buffer Stereo (interleaved) buffer to write audio to.
		\param numSamples The number of samples in the current block (i.e.
		buffer will contain 2x numSamples because it's stereo.
	 */
	void getAudio(float *buffer, int numSamples);

	///	Starts the synth playing.
	void start();
	///	Stops the synth playing.
	void stop();
  private:
	///	Helper method. Loads the loop sound file.
	void loadSoundFile();

	///	The loop buffer.
	std::vector<float> loopBuffer;
	///	The size of the loop buffer.
	int loopBufferSize;

	///	Struct representing one of the grains we use to play the loop buffer.
	struct Grain
	{
		///	The grain's current position within the buffer (0->1).
		float position;
		///	The direction of the grain.
		float direction;
		///	The grain's panning.
		float pan;
		///	Whether or not this grain is active.
		bool active;
	};

	///	The maximum number of Grains which may be active.
	enum
	{
		NumGrains = 8
	};

	///	The grains we use to play the loop buffer.
	Grain grains[NumGrains];

	///	Used to trigger a new Grain to be active.
	int nextOnset;

	///	For the fundamental sine tone.
	float sineX0;
	///	For the fundamental sine tone.
	float sineX1;
	///	For the fundamental sine tone.
	float sineCoeff;
	///	Used to fade the sine tone in and out.
	float sineFade;

	///	If we're currently playing or not.
	bool playing;
};

#endif
